<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/23/2009 *
 * ===================                          Last change: 10/23/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : sql-functions.php                                *
 * -------------------------------------------------------------------- *
 * Short description : SQL functions to handle queries                  *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : SQL-Funktionen fuer Queries                      *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Init SQLs array
function initSqls () {
	setSqlsArray(array());
}

// Checks wether the sqls array is initialized
function isSqlsInitialized () {
	return ((isset($GLOBALS['sqls'])) && (is_array($GLOBALS['sqls'])));
}

// Setter for SQLs array
function setSqlsArray ($SQLs) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'count()='.count($SQLs));
	$GLOBALS['sqls'] = (array) $SQLs;
}

// Remover for SQLs array
function unsetSqls () {
	unset($GLOBALS['sqls']);
}

// Getter for SQLs array
function getSqls () {
	return $GLOBALS['sqls'];
}

// Add an SQL to the list
function addSql ($sql) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, sprintf("sql=%s, count=%d", $sql, countSqls()));
	$GLOBALS['sqls']['generic'][] = (string) $sql;
}

// Merge SQLs together
function mergeSqls ($SQLs, $type = '') {
	// Should we merge full array or partial?
	if (empty($type)) {
		// Merge full array (may kill entries)
		setSqlsArray(merge_array(getSqls(), $SQLs));
	} else {
		// Merge sub array, so get it
		$array = getSqls();

		// Is the sub array there?
		if (isset($array[$type])) {
			// Then get it and merge it with the new one
			$array[$type] = merge_array($array[$type], $SQLs);
		} else {
			// Use new array
			$array[$type] = $SQLs;
		}

		// Call again..
		mergeSqls($array);
	}
}

// Counter for SQLs array
function countSqls () {
	// Default is false
	$count = '0';

	// Is the array there?
	if (isSqlsInitialized()) {
		// Then count it
		$count = count($GLOBALS['sqls']);
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, sprintf("count=%d", $count));
	} // END - if

	// Return it
	return $count;
}

// Checks wether the SQLs array is filled
function isSqlsValid () {
	//* DEBUG: */ debugOutput(__FUNCTION__ . ':' . intval(isSqlsInitialized()) . '/' . countSqls() . '/' . getCurrentExtensionName());
	return (
		(
			isSqlsInitialized()
		) && (
			countSqls() > 0
		)
	);
}

// [EOF]
?>
